breed [nodes node]
nodes-own [sources     ;; a list of id's of other nodes from which boolean arguments are taken.
           next-state  ;; a boolean, used for the synchronous update cycle
           function]   ;; a boolean function encoded as a list of booleans, of length 2 ^ (length sources)
                       ;;   For example, here's the encoding ('8') for arity-2 'AND':
                       ;;       Argument 0 (A) :  true false true  false 
                       ;;       Argument 1 (B) :  true true  false false
                       ;;       Encoding (A&B) : [true false false false] <-- encode 2 8 

globals [history] ;; a list of network states, used for detecting cycles

to startup setup end
to setup
  clear-all ask patches [set pcolor white]
  create-ordered-nodes -n-
    [set color black set label-color green fd 14]
  ask nodes
    [pick-random-sources
     pick-random-function]
  reset
end

to pick-random-sources
  ask my-in-links [die]
  set sources sort n-of -k- other nodes
  create-links-from turtle-set sources
end
   
to pick-random-function
  let my-fn-number random (2 ^ (2 ^ -k-))
  set function encode -k- my-fn-number
  if label-node-functions? [set label my-fn-number]
end

to reset
  ask nodes [ifelse random-float 1 < -p- [turn-black] [turn-white]]
  clear-plot setup-plots set history [] reset-ticks
end

to go       ;; in two 'half-steps' per tick, for synchrony
  ask nodes ;; first calculate my next state...
    [let args map black? sources
     set next-state eval-bfn function args]
  ask nodes ;; ...then actually update my state 
    [if-else next-state [turn-black] [turn-white]]
  tick
  if detect-cycles?
    [set history lput net-state history
     if member? (last history) butlast history
       [show-cycle stop]]
end

to show-cycle
  let cycle-start position (last history) history
  let cycle-states n-values (length history - cycle-start)
                            [? + cycle-start]
  let c length cycle-states - 1
  set-plot-pen-color red
  foreach butfirst cycle-states [plotxy 1.1 (0 - ?)]
  set-plot-pen-color black
   print (word "Cycle! length: " c " transient: " (ticks - c)
                     " code: " (sum map decode last-n c history))
end
        
to-report last-n [n a-list]
  let l length a-list
  report sublist a-list (l - n) l
end

to-report eval-bfn [fn-code args]
  let l length fn-code
  report ifelse-value (l = 1)
     [last fn-code]
     [ifelse-value first args
       [eval-bfn (sublist fn-code 0 (l / 2)) (butfirst args)]
       [eval-bfn (sublist fn-code (l / 2) l) (butfirst args)]]
end

to-report encode [r n] ;; 'r' for arity
  report pad-to-length (2 ^ r) binary-list n
end

to-report odd? [n] report n mod 2 = 1 end

to-report binary-list [n] report
;; integer n represented as a list of booleans, where item i is the (2^i)'s bit.
;; to read the list (backward) as a 'normal' binary number replace true by 1, false by 0
  ifelse-value (n < 1)
    [ [] ]  [lput (odd? n) binary-list (floor (n / 2))]
end

to-report pad-to-length [n a-list]
  if length a-list > n [error "list too long!"]
  report ifelse-value (length a-list >= n)
           [a-list]  [pad-to-length n (fput false a-list)]
end

to-report decode [a-bool-list] ;; decimal representation
  report ifelse-value (empty? a-bool-list) [0]
           [2 ^ (length a-bool-list - 1)
              * ifelse-value (first a-bool-list) [1] [0]
            + decode butfirst a-bool-list]
end

to-report to-binary [a-bool-list]  ;; true/false --> 1/0
  report map [ifelse-value ? [1] [0]] a-bool-list
end

to-report from-binary [a-number-list] ;; 1/0 --> true/false
  report map [? = 1] a-number-list
end

to set-net-state [a-bool-list] ;; the list should have -N- values
  let i 0
  foreach a-bool-list [ask node i [ifelse ? [turn-black] [turn-white]]
                       set i i + 1]
end

to flip ifelse black? self [turn-white] [turn-black] end
to turn-black set shape "circle"   end ;; of course, you could use 'real'
to turn-white set shape "circle 2" end ;; colors instead, if you prefer!
to-report black? [a-node] report [shape] of a-node = "circle" end
to-report net-state report map [black? ?] sort nodes end
;; non-black nodes are presumed white, since it's a boolean network
@#$#@#$#@
GRAPHICS-WINDOW
204
10
813
640
16
16
18.152
1
14
1
1
1
0
0
0
1
-16
16
-16
16
0
0
1
ticks
30.0

BUTTON
6
163
86
196
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
2
10
201
43
-N-
-N-
1
80
10
1
1
nodes
HORIZONTAL

BUTTON
122
201
199
234
NIL
go
T
1
T
OBSERVER
NIL
G
NIL
NIL
1

BUTTON
7
201
117
234
go once
go
NIL
1
T
OBSERVER
NIL
1
NIL
NIL
1

PLOT
5
238
200
602
network states over time
node states
<-- ticks --
0.0
1.0
-100.0
0.0
true
false
"set-plot-pen-mode 2" "let state net-state ;; to avoid re-calculating\nforeach n-values count turtles [?] [if item ? state [plotxy (? / -N-) (0 - ticks)]]"
PENS
"default" 1.0 0 -16777216 true "" ""

SLIDER
3
48
201
81
-K-
-K-
1
3
1
1
1
in-links per node
HORIZONTAL

SLIDER
4
85
201
118
-P-
-P-
0
1
0.5
0.01
1
initially black
HORIZONTAL

SWITCH
4
124
201
157
label-node-functions?
label-node-functions?
0
1
-1000

BUTTON
91
163
200
196
NIL
reset
NIL
1
T
OBSERVER
NIL
R
NIL
NIL
1

BUTTON
140
606
200
639
clear
clear-plot reset-ticks setup-plots
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SWITCH
8
606
136
639
detect-cycles?
detect-cycles?
0
1
-1000

@#$#@#$#@
## KAUFFMAN'S 'CLASSIC' RANDOM BOOLEAN NETWORK

Also known as NK networks, these discrete dynamical systems were first proposed by Stuart Kauffman in 1969 as an extremely simplified model of a genetic regulatory network. A 'classical' RBN is a set of _N_ Boolean variables, each of which is assigned a randomly generated Boolean function of arity _K_ and given links from _K_ other randomly selected variables in the set, which provide the arguments or inputs for the function. Nodes in this network represent both the function (which does not change) and its value at a particular time. The connectivity of the network remains fixed. All nodes update their states simultaneously, once per tick, each evaluating its own function with arguments taken (in order, clockwise from the top) from its _K_ neighbors, which are shown by the arrows.

## IMPORTANT CONCEPTS

Boolean functions of finite arity are enumerable. Because there are 2<sup>_K_</sup> possible arguments for a Boolean function of arity _K_, each of which the function must evaluate to one of 2 values, there are <sub>2</sub>2<sup>_K_</sup> such functions. We can number them in a very natural way by lining up, in order, all the possible arguments (if it helps, think of them as the numbers from 0 to 2<sup>_K_</sup> – 1, written in binary) and recording as a bit the value which the function assigns each argument. Reading off these bits in order, we have a binary number between 0 and <sub>2</sub>2<sup>_K_</sup> – 1, inclusive. (This is the same numbering scheme conventionally used for one-dimensional cellular automata.) We can also represent the state of the network as a binary number between 0 and 2<sup>_N_</sup> – 1, which is helpful in keeping track of its dynamics.

## THINGS TO NOTICE AND TRY

The plot gives a way of displaying, in a compressed form, the evolution of the system. Each state is plotted as horizontal row of dots; time reads down the plot. Attractors appear as repeated patterns: solid vertical stripes are fixed points. If you turn the `detect-cycles` switch on, the model will stop automatically when a new state is generated that is the same as a previous state, and indicate the cycle with a red line. (For a fixed point attractor, the indicator will be just a red dot.) It will print out the length and code number of the cycle in the Command Center. The code is just the sum, in decimal, of all the states in the cycle.

Because the system is deterministic and has only 2<sup>_N_</sup> distinct states, it must fall into a cycle of length less than 2<sup>_N_</sup>. For a given network, how long is a typical cycle, and how quickly is it attained? How many different cyclic attractors are there? How many fixed point attractors? How do the different parameters affect the dynamics?

You can detect and highlight cycles automatically by setting the `detect-cycles?` switch. This will slow down the program, though, because each new state must be compared with all previous states.

## EXTEND THIS MODEL!

This model is deliberately minimal, because there are so many possible extensions and variations to the NK model. Here are a few you might want to investigate: 

  * Alter the network topology
  * Make the update asynchronous
  * Have a node's next state depend in part on its own current state
  * Give each node the same function, or one of a subset of possible functions
  * Allow functions of different arities, so that instead of a single global K, there is a 'little k' for each node

Can you think of others? How do they affect the behavior of the system?


## CREDITS AND REFERENCES

Kauffman, S. A. (1969). Metabolic stability and epigenesis in randomly constructed genetic nets. Journal of Theoretical Biology, 22:437-467.

NetLogo is © Uri Wilensky and the Center for Connected Learning of Northwestern University.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -1 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
2.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 135 180
Line -7500403 true 150 150 165 180

@#$#@#$#@
0
@#$#@#$#@
